package com.sxtvs.open.api.odata.service;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.http.HttpUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sxtvs.open.api.odata.dao.DouyinAccountMapper;
import com.sxtvs.open.api.odata.dto.douyin.DouyinAccessTokenResponse;
import com.sxtvs.open.api.odata.dto.douyin.DouyinRefreshTokenResponse;
import com.sxtvs.open.api.odata.dto.douyin.DouyinUserInfoResponse;
import com.sxtvs.open.api.odata.dto.douyin.videolist.Data;
import com.sxtvs.open.api.odata.dto.douyin.videolist.DouyinVideoListResponse;
import com.sxtvs.open.api.odata.entity.DouyinAccount;
import com.sxtvs.open.core.conf.Constant;
import com.sxtvs.open.core.lock.SegmentLock;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.retry.annotation.Backoff;
import org.springframework.retry.annotation.Retryable;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.util.Date;
import java.util.HashMap;

@Service
@Slf4j
public class DouyinAccountService extends ServiceImpl<DouyinAccountMapper, DouyinAccount> {

    private final SegmentLock<String> segmentLock = new SegmentLock<>(128, false);

    @Autowired
    @Qualifier("douyinRestTemplate")
    private RestTemplate restTemplate;

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private QrcodeLogServiceImpl qrcodeLogService;

    private void renewRefreshToken(DouyinAccount douyinAccount) {

        HashMap<String, Object> param = new HashMap<String, Object>() {{
            this.put("client_key", Constant.douyinClientKey);
            this.put("refresh_token", douyinAccount.getRefreshToken());
        }};

        String body = HttpUtil.post("https://open.douyin.com/oauth/renew_refresh_token/", param);
        String refreshToken;
        try {
            log.info("renew_refresh_token {}", body);
            DouyinRefreshTokenResponse douyinRefreshTokenResponse = objectMapper.readValue(body, DouyinRefreshTokenResponse.class);
            refreshToken = douyinRefreshTokenResponse.getData().getRefreshToken();
            if (refreshToken == null) {
                log.error("{}", douyinAccount.getNickName());
                return;
            }
        } catch (JsonProcessingException e) {
            log.error(e.getMessage(), e);
            return;
        }
        DateTime now = new DateTime();
        douyinAccount.setRefreshToken(refreshToken);
        douyinAccount.setRtExpireTime(DateUtil.offsetDay(now, 30));
        douyinAccount.setUpdateTime(now.toJdkDate());
        this.updateById(douyinAccount);
    }

    private void renewAccessToken(DouyinAccount douyinAccount) {
        HashMap<String, Object> param = new HashMap<String, Object>() {{
            this.put("client_key", Constant.douyinClientKey);
            this.put("grant_type", "refresh_token");
            this.put("refresh_token", douyinAccount.getRefreshToken());
        }};
        String body = HttpUtil.post("https://open.douyin.com/oauth/refresh_token/", param);
        String accessToken;
        try {
            DouyinAccessTokenResponse tokenResponse = objectMapper.readValue(body, DouyinAccessTokenResponse.class);
            accessToken = tokenResponse.getData().getAccessToken();
            if (accessToken == null) {
                log.error("{} {}", douyinAccount.getNickName(), body);
                return;
            }
        } catch (JsonProcessingException e) {
            log.error(e.getMessage(), e);
            return;
        }
        DouyinAccessTokenResponse response = restTemplate.getForObject("https://open.douyin.com/oauth/refresh_token/?" +
                        "client_key={client_key}&grant_type={grant_type}&refresh_token={refresh_token}",
                DouyinAccessTokenResponse.class, param);

        DateTime now = new DateTime();
        douyinAccount.setAccessToken(accessToken);
        douyinAccount.setAtExpireTime(DateUtil.offsetDay(now, 15));
        douyinAccount.setUpdateTime(now.toJdkDate());
        this.updateById(douyinAccount);
        log.info("renewAccessToken {}", response);

    }

    @Retryable(value = RestClientException.class,
            backoff = @Backoff(delay = 5000L, multiplier = 2))
    public DouyinAccount upsert(String code) {
        HashMap<String, String> param = new HashMap<String, String>() {{
            this.put("client_key", Constant.douyinClientKey);
            this.put("client_secret", Constant.douyinClientSecret);
            this.put("code", code);
        }};
        DouyinAccessTokenResponse douyinAccessTokenResponse = restTemplate.getForObject(
                "https://open.douyin.com/oauth/access_token/?" +
                        "client_key={client_key}&client_secret={client_secret}&" +
                        "grant_type=authorization_code&code={code}",
                DouyinAccessTokenResponse.class, param);
        DouyinAccessTokenResponse.Data data = douyinAccessTokenResponse.getData();
        String openId = data.getOpenId();
        String accessToken = data.getAccessToken();
        String refreshToken = data.getRefreshToken();

        param.clear();
        param.put("access_token", accessToken);
        param.put("open_id", openId);
        DouyinUserInfoResponse douyinUserInfoResponse = restTemplate.getForObject("https://open.douyin.com/oauth/userinfo/?" +
                        "access_token={access_token}&open_id={open_id}",
                DouyinUserInfoResponse.class, param);


        DouyinAccount douyinAccount = this.getById(openId);

        DateTime now = new DateTime();
        log.info("getById douyinAccount {} ", douyinAccount);
        String nickname = douyinUserInfoResponse.getData().getNickname();
        qrcodeLogService.writeLog("抖音",openId,nickname);
        if (douyinAccount == null) {
            douyinAccount = DouyinAccount.builder()
                    .openId(openId)
                    .atExpireTime(DateUtil.offsetDay(now, 15))
                    .rtExpireTime(DateUtil.offsetDay(now, 30))
                    .accessToken(accessToken)
                    .refreshToken(refreshToken)
                    .nickName(nickname)
                    .updateTime(now.toJdkDate())
                    .createTime(now.toJdkDate())
                    .build();
            this.save(douyinAccount);
            log.info("{} 新增成功", nickname);
        } else {
            douyinAccount.setAtExpireTime(DateUtil.offsetDay(now, 15));
            douyinAccount.setRtExpireTime(DateUtil.offsetDay(now, 30));
            douyinAccount.setAccessToken(accessToken);
            douyinAccount.setRefreshToken(refreshToken);
            douyinAccount.setNickName(nickname);
            douyinAccount.setUpdateTime(now.toJdkDate());
            this.updateById(douyinAccount);
            log.info("douyinAccount {}  更新成功", douyinAccount);
        }

        return douyinAccount;

    }

    @Retryable(value = RestClientException.class,
            backoff = @Backoff(delay = 5000L, multiplier = 2))
    public Data videoList(String openId, String cursor) {
        DouyinAccount douyinAccount = this.getById(openId);
        HashMap<String, String> param = new HashMap<String, String>() {{
            this.put("open_id", douyinAccount.getOpenId());
            this.put("access_token", douyinAccount.getAccessToken());
            this.put("cursor", cursor);
            this.put("count", "10");
        }};
        DouyinVideoListResponse douyinVideoListResponse = restTemplate.getForObject(
                "https://open.douyin.com/video/list/?open_id={open_id}&access_token={access_token}&" +
                        "cursor={cursor}&count={count}",
                DouyinVideoListResponse.class, param);
        if (douyinVideoListResponse == null) {
            throw new RuntimeException();
        }

        return douyinVideoListResponse.getData();
    }


    @Retryable(value = RestClientException.class,
            backoff = @Backoff(delay = 5000L, multiplier = 2))
    public void refresh() {
        Date rtExpireTime = new DateTime()
                .offset(DateField.DAY_OF_YEAR, 7)
                .toJdkDate();
        Date expireTime = new DateTime()
                .offset(DateField.DAY_OF_YEAR, 1)
                .offset(DateField.HOUR_OF_DAY, 12)
                .toJdkDate();


        //检测快过期的rt
        for (DouyinAccount douyinAccount : this.lambdaQuery()
                .le(DouyinAccount::getRtExpireTime, rtExpireTime)
                .list()) {
            renewRefreshToken(douyinAccount);
        }

        //检测快过期的at
        for (DouyinAccount douyinAccount : this.lambdaQuery()
                .le(DouyinAccount::getAtExpireTime, expireTime)
                .list()) {
            renewAccessToken(douyinAccount);
        }


    }


}
