package com.sxtvs.open.api.odata.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.http.HttpUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sxtvs.open.api.odata.dao.KuaishouAccountMapper;
import com.sxtvs.open.api.odata.dto.kuaishou.KuaiShouVideoResponse;
import com.sxtvs.open.api.odata.dto.kuaishou.KuaishouAccessTokenResponse;
import com.sxtvs.open.api.odata.dto.kuaishou.KuaishouUserInfoResponse;
import com.sxtvs.open.api.odata.entity.KuaishouAccount;
import com.sxtvs.open.api.odata.entity.PlatformData;
import com.sxtvs.open.core.conf.Constant;
import com.sxtvs.open.core.lock.SegmentLock;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.retry.annotation.Backoff;
import org.springframework.retry.annotation.Retryable;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class KuaishouAccountService extends ServiceImpl<KuaishouAccountMapper, KuaishouAccount> {

    private final SegmentLock<String> segmentLock = new SegmentLock<>(128, false);

    @Autowired
    @Qualifier("kuaishouRestTemplate")
    private RestTemplate restTemplate;

    @Autowired
    private PlatformDataService platformDataService;

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private QrcodeLogServiceImpl qrcodeLogService;


    @Retryable(value = RestClientException.class,
            backoff = @Backoff(delay = 5000L, multiplier = 2))
    public KuaishouAccount upsert(String code) {
        HashMap<String, String> param = new HashMap<String, String>() {{
            this.put("app_id", Constant.kuaishouClientKey);
            this.put("app_secret", Constant.kuaishouClientSecret);
            this.put("code", code);
        }};
        KuaishouAccessTokenResponse data = restTemplate.getForObject(
                "https://open.kuaishou.com/oauth2/access_token?" +
                        "app_id={app_id}&app_secret={app_secret}&" +
                        "grant_type=authorization_code&code={code}",
                KuaishouAccessTokenResponse.class, param);
        if (data == null) {
            log.error("code {} getAccessToken", code);
            return null;
        }
        log.info("KuaishouAccessTokenResponse {}", data);
        String openId = data.getOpenId();
        String accessToken = data.getAccessToken();
        String refreshToken = data.getRefreshToken();

        param.clear();
        param.put("access_token", accessToken);
        param.put("app_id", Constant.kuaishouClientKey);
        KuaishouUserInfoResponse kuaishouUserInfoResponse = restTemplate.getForObject("https://open.kuaishou.com/openapi/user_info?" +
                        "access_token={access_token}&app_id={app_id}",
                KuaishouUserInfoResponse.class, param);

        if (kuaishouUserInfoResponse == null) {
            log.error("param {} getUserInfo", param);
            return null;
        }



        KuaishouAccount kuaishouAccount = this.getById(openId);

        DateTime now = new DateTime();
        log.info("getById kuaishouAccount {} ", kuaishouAccount);
        String nickname = kuaishouUserInfoResponse.getUserInfo().getName();
        qrcodeLogService.writeLog("快手",openId,nickname);
        if (kuaishouAccount == null) {
            KuaishouAccount account = KuaishouAccount.builder()
                    .openId(openId)
                    .atExpireTime(DateUtil.offsetSecond(now, data.getExpiresIn()))
                    .rtExpireTime(DateUtil.offsetSecond(now, data.getRefreshTokenExpiresIn()))
                    .accessToken(accessToken)
                    .refreshToken(refreshToken)
                    .nickName(nickname)
                    .updateTime(now.toJdkDate())
                    .createTime(now.toJdkDate())
                    .build();
            this.save(account);
            log.info("{} 新增成功", nickname);
        } else {
            kuaishouAccount.setAtExpireTime(DateUtil.offsetSecond(now, data.getExpiresIn()));
            kuaishouAccount.setRtExpireTime(DateUtil.offsetSecond(now, data.getRefreshTokenExpiresIn()));
            kuaishouAccount.setAccessToken(accessToken);
            kuaishouAccount.setRefreshToken(refreshToken);
            kuaishouAccount.setNickName(nickname);
            kuaishouAccount.setUpdateTime(now.toJdkDate());
            this.updateById(kuaishouAccount);
            log.info("kuaishouAccount {}  更新成功", kuaishouAccount);
        }


        return kuaishouAccount;

    }

    @Retryable(value = RestClientException.class,
            backoff = @Backoff(delay = 5000L, multiplier = 2))
    public void refresh() {

        Date expireTime = new DateTime()
                .offset(DateField.DAY_OF_YEAR, 7)
                .toJdkDate();

        //检测快过期的rt
        for (KuaishouAccount kuaishouAccount : this.lambdaQuery()
                .le(KuaishouAccount::getRtExpireTime, expireTime)
                .list()) {
            renewAccessToken(kuaishouAccount);
        }


        expireTime = new DateTime()
                .offset(DateField.MINUTE, 30)
                .toJdkDate();

        //检测快过期的at
        for (KuaishouAccount kuaishouAccount : this.lambdaQuery()
                .le(KuaishouAccount::getAtExpireTime, expireTime)
                .list()) {
            renewAccessToken(kuaishouAccount);
        }


    }

    @SneakyThrows
    private void renewAccessToken(KuaishouAccount kuaishouAccount) {


        String body = HttpUtil.get("https://open.kuaishou.com/oauth2/refresh_token?" +
                "refresh_token=" + kuaishouAccount.getRefreshToken() + "&app_id=" + Constant.kuaishouClientKey + "&" +
                "app_secret=" + Constant.kuaishouClientSecret + "&grant_type=refresh_token");
        log.info("renewAccessToken {}", body);
        KuaishouAccessTokenResponse response = objectMapper.readValue(body, KuaishouAccessTokenResponse.class);

        if (response.getResult() != null && 1 != response.getResult()) {
            return;
        }


        DateTime now = new DateTime();
        kuaishouAccount.setRefreshToken(response.getRefreshToken());
        kuaishouAccount.setRtExpireTime(DateUtil.offsetSecond(now, response.getRefreshTokenExpiresIn()));
        kuaishouAccount.setAccessToken(response.getAccessToken());
        kuaishouAccount.setAtExpireTime(DateUtil.offsetSecond(now, response.getExpiresIn()));
        kuaishouAccount.setUpdateTime(now.toJdkDate());
        this.updateById(kuaishouAccount);
    }


    @Retryable(value = RestClientException.class,
            backoff = @Backoff(delay = 5000L, multiplier = 2))
    public List<KuaiShouVideoResponse.VideoList> videoList(String openId, String cursor) {
        KuaishouAccount kuaishouAccount = this.getById(openId);
        HashMap<String, String> param = new HashMap<String, String>() {{
            this.put("app_id", Constant.kuaishouClientKey);
            this.put("access_token", kuaishouAccount.getAccessToken());
            this.put("count", "20");
        }};
        String url = "https://open.kuaishou.com/openapi/photo/list?app_id={app_id}&access_token={access_token}&" +
                "count={count}";
        if (cursor != null) {
            url += "&cursor=" + cursor + "";
        }
        KuaiShouVideoResponse kuaiShouVideoResponse = restTemplate.getForObject(
                url,
                KuaiShouVideoResponse.class, param);
        if (kuaiShouVideoResponse == null) {
            throw new RuntimeException();
        }
        return kuaiShouVideoResponse.getVideoList();
    }

    private void saveKuaiShouData(String dt, KuaishouAccount kuaishouAccount) {
        String openId = kuaishouAccount.getOpenId();

        List<KuaiShouVideoResponse.VideoList> videoLists = videoList(openId, null);
        if (CollUtil.isEmpty(videoLists)) {
            return;
        }
        List<KuaiShouVideoResponse.VideoList> res = new ArrayList<>(videoLists);
        while (CollUtil.isNotEmpty(videoLists)) {
            videoLists = videoList(openId, videoLists.get(videoLists.size() - 1).getPhotoId());
            res.addAll(videoLists);
        }

        List<PlatformData> dataList = res.stream()
                .map(item -> {
                    PlatformData kuaishouData = new PlatformData();
                    kuaishouData.setId(item.getPhotoId() + dt);
                    kuaishouData.setItemId(item.getPhotoId());
                    kuaishouData.setDt(dt);
                    kuaishouData.setTitle(item.getCaption());
                    kuaishouData.setOpenId(openId);
                    kuaishouData.setDiggCount(item.getLikeCount());
                    kuaishouData.setPlayCount(item.getViewCount());
                    kuaishouData.setDownloadCount(0);
                    kuaishouData.setForwardCount(0);
                    kuaishouData.setShareCount(0);
                    kuaishouData.setCommentCount(item.getCommentCount());
                    kuaishouData.setPlatform("kuaishou");
                    kuaishouData.setNickName(kuaishouAccount.getNickName());
                    kuaishouData.setPublishTime(new Date(item.getCreateTime()));
                    return kuaishouData;
                })
                .collect(Collectors.toList());

        platformDataService.removeByIds(dataList.stream().map(PlatformData::getId)
                .collect(Collectors.toList()));

        platformDataService.saveBatch(dataList);
    }

    public void saveAllAccountData() {
        List<KuaishouAccount> kuaishouAccounts = list();
        String dt = new DateTime().toString("yyyyMMdd");

        for (KuaishouAccount kuaishouAccount : kuaishouAccounts) {
            try {
                saveKuaiShouData(dt, kuaishouAccount);
            } catch (Exception e) {
                String nickName = kuaishouAccount.getNickName();
                log.error(nickName + " error", e);
            }
        }

    }


}
