package com.sxtvs.open.core.advice;

import com.sxtvs.open.core.sls.AliyunLogger;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.multipart.MaxUploadSizeExceededException;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

import java.lang.annotation.Annotation;
import java.util.List;

/**
 * 这个类不要加业务逻辑
 */
@RestControllerAdvice
@Slf4j
public class APIResponseAdvice implements ResponseBodyAdvice<Object> {

    @Autowired
    private AliyunLogger logger;

    //自动处理APINoDataResponse，包装为APINoDataResponse
    @ExceptionHandler(BizException.class)
    public APIResponse handleBizException(HttpServletRequest request, BizException ex, HttpServletResponse response) {
        logger.warn("error", ex);
        APIResponse apiNoDataResponse = new APIResponse();
        apiNoDataResponse.setCode(ex.getErrorCode());
        apiNoDataResponse.setMessage(ex.getErrorMessage());
        return apiNoDataResponse;
    }

    @ExceptionHandler(MaxUploadSizeExceededException.class)
    public APIResponse handleException(HttpServletRequest request, MaxUploadSizeExceededException ex, HttpServletResponse response) {
        logger.warn("error", ex);
        APIResponse apiResponse = new APIResponse();
        apiResponse.setCode(-1);
        apiResponse.setMessage("文件长度过长");
        return apiResponse;
    }


    /**
     * 拦截未知的运行时异常
     */
    @ExceptionHandler(Throwable.class)
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    public APIResponse serverError(Throwable ex) {
        logger.error("error", ex);
        APIResponse apiResponse = new APIResponse();
        apiResponse.setCode(-1);
        apiResponse.setMessage("服务器运行异常");
        return apiResponse;
    }

    @ExceptionHandler(HttpMessageNotReadableException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public APIResponse messageParseException(HttpMessageNotReadableException ex) {
        logger.warn("error", ex);
        APIResponse apiResponse = new APIResponse();
        apiResponse.setCode(-1);
        apiResponse.setMessage("服务器运行异常");
        return apiResponse;
    }

    @ExceptionHandler(MethodArgumentNotValidException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public APIResponse validationBodyException(MethodArgumentNotValidException exception) {
        StringBuilder sb = new StringBuilder();
        BindingResult result = exception.getBindingResult();
        StringBuilder message = new StringBuilder();
        if (result.hasErrors()) {
            List<ObjectError> errors = result.getAllErrors();
            errors.forEach(p -> {
                FieldError fieldError = (FieldError) p;
                sb.append("Data check failure : object{").append(fieldError.getObjectName()).append("},field{")
                        .append(fieldError.getField()).append("},errorMessage{")
                        .append(fieldError.getDefaultMessage()).append("}");

                message.append(fieldError.getDefaultMessage()).append(" ");
            });
        } else {
            message.append("参数异常");
        }
        logger.warn("error", sb.toString());
        APIResponse apiResponse = new APIResponse();
        apiResponse.setCode(-1);
        apiResponse.setMessage(message.toString());
        return apiResponse;
    }


    //仅当方法或类没有标记@NoAPIResponse才自动包装
    @Override
    public boolean supports(MethodParameter returnType, Class converterType) {
        for (Annotation methodAnnotation : returnType.getMethodAnnotations()) {
            if (methodAnnotation.annotationType() == NoAPIResponse.class) {
                return false;
            }
        }
        return returnType.getParameterType() != APIResponse.class
                && AnnotationUtils.findAnnotation(returnType.getDeclaringClass(), NoAPIResponse.class) == null;
    }

    //自动包装外层APIResposne响应
    @Override
    public Object beforeBodyWrite(Object body,
                                  MethodParameter returnType,
                                  MediaType selectedContentType,
                                  Class<? extends HttpMessageConverter<?>> selectedConverterType,
                                  ServerHttpRequest request, ServerHttpResponse response) {


        ResponseMsg responseMsg = returnType.getMethodAnnotation(ResponseMsg.class);
        String msg = "OK";

        if (null != responseMsg) {
            msg = responseMsg.value();
        }

        APIResponse apiResponse = new APIResponse();
        apiResponse.setCode(0);
        apiResponse.setMessage(msg);
        apiResponse.setData(body);
        return apiResponse;

    }

}
